----------------------------------------------------------------
--  Title     : Mapper.vhd
--  Function  : Memory mapper
--  Date      : 28th,August,2000
--  Revision  : 1.01
--  Author    : Kazuhiro TSUJIKAWA (ESE Artists' factory)
----------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity mapper is
  port(
    pSltClk     : IN std_logic;
    pSltRst_n   : IN std_logic;
    pSltSltsl_n : IN std_logic;
    pSltIorq_n  : IN std_logic;
    pSltRd_n    : IN std_logic;
    pSltWr_n    : IN std_logic;
    pSltAdr     : IN std_logic_vector(15 downto 0);
    pSltDat     : INOUT std_logic_vector(7 downto 0);
   
    pRamCeX_n   : OUT std_logic;
    pRamOeX_n   : OUT std_logic;
    pRamWeX_n   : OUT std_logic;
    pRamAdrX    : OUT std_logic_vector(18 downto 0);
    pRamDatX    : INOUT std_logic_vector(7 downto 0)

   
   
  );
end mapper;

architecture RTL of mapper is

  signal pSltClk_n   : std_logic;
  signal DevHit      : std_logic;

  signal RamAdrEx    : std_logic;
  signal RamBank0    : std_logic_vector(7 downto 0);
  signal RamBank1    : std_logic_vector(7 downto 0);
  signal RamBank2    : std_logic_vector(7 downto 0);
  signal RamBank3    : std_logic_vector(7 downto 0);

begin

  ----------------------------------------------------------------
  -- Dummy pin
  ----------------------------------------------------------------
 

  pSltClk_n <= not pSltClk;

 

 
  ----------------------------------------------------------------
  -- Slot access control
  ----------------------------------------------------------------
  process(pSltClk_n, pSltRst_n, pSltIorq_n, pSltSltsl_n, pSltRd_n, pSltWr_n)

    variable DevAcs0 : std_logic;
    variable DevAcs1 : std_logic;

  begin

    if ((pSltIorq_n = '0' or pSltSltsl_n = '0') and (pSltRd_n = '0' or pSltWr_n = '0')) then
      DevAcs0 := '1';
    else
      DevAcs0 := '0';
    end if;

    if (DevAcs0 = '1' and DevAcs1 = '0') then
      DevHit <= '1';
    else
      DevHit <= '0';
    end if;

    if (pSltRst_n = '0') then
      DevAcs1 := '0';
    elsif (pSltClk_n'event and pSltClk_n = '1') then
      DevAcs1 := DevAcs0;
    end if;

  end process;

  ----------------------------------------------------------------
  -- Mapper bank register access
  ----------------------------------------------------------------
  process(pSltClk_n, pSltRst_n)

  begin

    if (pSltRst_n = '0') then

      RamBank0   <= (others => '0');
      RamBank1   <= (others => '0');
      RamBank2   <= (others => '0');
      RamBank3   <= (others => '0');
		

    elsif (pSltClk_n'event and pSltClk_n = '1') then

      -- I/O port access on FC-FFh ... Mapper bank resister write
      if (DevHit = '1' and pSltIorq_n = '0' and pSltWr_n = '0' and pSltAdr(7 downto 2) = "111111") then
        case pSltAdr(1 downto 0) is
          when "00"   => RamBank0 <= pSltDat;
          when "01"   => RamBank1 <= pSltDat;
          when "10"   => RamBank2 <= pSltDat;
          when others => RamBank3 <= pSltDat;
        end case;
      end if;

    end if;

  end process;

  ----------------------------------------------------------------
  -- Memory mapper memory access
  ----------------------------------------------------------------
  RamAdrEx  <= RamBank0(5) when pSltAdr(15 downto 14) = "00" else
               RamBank1(5) when pSltAdr(15 downto 14) = "01" else
               RamBank2(5) when pSltAdr(15 downto 14) = "10" else
               RamBank3(5);

  pRamCeX_n <= '0' when pSltSltsl_n = '0' and pSltRst_n = '1' and RamAdrEx = '0' else '1';
 
  pRamOeX_n <= '0' when pSltSltsl_n = '0' and pSltRd_n = '0' and pSltRst_n = '1' else '1';
  
  pRamWeX_n <= '0' when pSltSltsl_n = '0' and pSltWr_n = '0' and pSltRst_n = '1' else '1';
  
  pRamAdrX  <= RamBank0(4 downto 0) & pSltAdr(13 downto 0) when pSltAdr(15 downto 14) = "00" else
               RamBank1(4 downto 0) & pSltAdr(13 downto 0) when pSltAdr(15 downto 14) = "01" else
               RamBank2(4 downto 0) & pSltAdr(13 downto 0) when pSltAdr(15 downto 14) = "10" else
               RamBank3(4 downto 0) & pSltAdr(13 downto 0);
  

  pRamDatX  <= pSltDat when pSltSltsl_n = '0' and pSltWr_n = '0' and RamAdrEx = '0' else (others => 'Z');
  

  --pSltDat   <= pRamDatX when pSltSltsl_n = '0' and pSltRd_n = '0' and RamAdrEx = '0' else
  --             (others => 'Z');

end RTL;
